%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright ?American Optimal Decisions, Inc. 2007-2016.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and ?No Warranty?  statements are included with all reproductions      |
%and derivative works.                                                  |
%=======================================================================|
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|
% This program implements  the Case Study CS_Cell_Towers_allocation


% 0. Clear command window and workspace
clc; clear


% 1. Load Bipartite Graph

disp(' ');
disp(' Part 1. Load Bipartite Graph and generate Cell Towers Problem Statement ')
disp(' ')

% Type a name of file containing Bipartite Graph

%load matrix_of_Bipartite_Graph.txt;
%matrix_BiGraph=matrix_of_Bipartite_Graph;
  % or
load matrix_Bipartite_Graph_10x10x041.txt;              % greed 10x10 with 41 houses
matrix_BiGraph=matrix_Bipartite_Graph_10x10x041;
  % or
%load matrix_Bipartite_Graph_10x10x041_uns.txt;         % greed 10x10 with 41 houses. Arbitrary indexes
%matrix_BiGraph=matrix_Bipartite_Graph_10x10x041_uns;
  % or
%load matrix_Bipartite_Graph_300x300x05.txt;            % greed 300x300 with 50%of houses
%matrix_BiGraph=matrix_Bipartite_Graph_300x300x05;

% One part of Bipartite Graph describes consumers, other part - suppliers. 
% Set parameter Place_of_consumers = 1 (below) if consumers are in left part. Otherwise set Place_of_consumers ~= 1.
% Every row of the matrix of the Bipartite Graph contains two integers.
% One integer is a consumer # (or user, or house), the other integer is a supplier # (or Cell Tower, or facility, or server).
% Integer numbers may be arbitrary in range [-1e6:+1e6].
% A row is included in the matrix only if the consumer can be served by a supplier.
% An order of rows (connections) is arbitrary.

Place_of_consumers=1;


% 2. Choose parameters for problems

Max_unserved_level=0.3;         % maximal fraction of unserved consumers (0<Max_unserved_level<1)
Min_covering_to_be_served=1;    % minimum level of redundant coverage by suppliers for a consumer (integer 1,2,3,...)
Number_of_suppliers=6;          % maximum number of suppliers in the third problem (integer 1,2,...,ks)


% 3. Get consumers and suppliers indices and make working BiGraph BG 
tic

I=size(matrix_BiGraph,2);     % number of columns (should be 2)
J=size(matrix_BiGraph,1);     % number of connections between left and right parts of the BiGraph

if Place_of_consumers==1, 
  Consumers=matrix_BiGraph(:,1); Suppliers=matrix_BiGraph(:,2);
else
  Suppliers=matrix_BiGraph(:,1); Consumers=matrix_BiGraph(:,2); 
end
    
% Place consumers in the first column and suppliers in the second column in working matrix BG of BiGraph
BG=[Consumers Suppliers];

tic
% Find number and indices of different consumers and suppliers
[nc,indc]=sort(Consumers);
kc=1; BG(indc(1),1)=1; icp=nc(1);
for j=2:J; ic=nc(j);
  if ic>icp, kc=kc+1; nc(kc)=ic; icp=ic; end
  BG(indc(j),1)=kc;
end
% Consumers = vector of indices of consumers
% kc = number of different consumers
Consumers=nc(1:kc);
toc

tic
[ns,inds]=sort(Suppliers);
ks=1; BG(inds(1),2)=1; isp=ns(1); 
for j=2:J; is=ns(j);
  if is>isp, ks=ks+1; ns(ks)=is; isp=is; end
  BG(inds(j),2)=ks;
end
% Suppliers = vector of indices of suppliers
% ks = number of different suppliers
Suppliers=ns(1:ks);
toc

tic
% Sorting connection in BiGraph: first by consumers, second by suppliers
d=max(max(BG));
if d>=1e7, disp(''); disp(' Too many consumers ot suppliers in BiGraph. Stop '); break; end
ncs=BG(:,1)*1e7+BG(:,2);
[nc,ind]=sort(ncs);
toc

tic
% Change the order of rows in matrix BG according to previous sorting
BG=BG(ind(1:J),:);
toc

clear ncs nc ns d ic icp is isp ind indc inds matrix_BiGraph

% 4. Prepare matrices for PSG solver

tic
% Generate two matrices for mpsg solver
count = 1;
BG=[BG ones(J,1)]; BGS=sparse(BG(:,1),BG(:,2),BG(:,3));

% 5. Solve 3 problems
disp(' ')
disp(' Part 2. Solve 3 problems ')
disp(' ')

tic;
[solution_Pr_in_constraint, output_Pr_in_constraint] = problem_Pr_in_constraint(BGS,[],ones(1,ks),[],Max_unserved_level,-Min_covering_to_be_served);
disp(' '); toc;  disp(' ')

tic;
[solution_VaR_in_constraint, output_VaR_in_constraint] = problem_VaR_in_constraint(BGS,[],ones(1,ks),[],-Min_covering_to_be_served,1-Max_unserved_level);
disp(' '); toc;  disp(' ')

tic;
[solution_Pr_in_objective, output_Pr_in_objective] = problem_Pr_in_objective(BGS,[],ones(1,ks),[],-Min_covering_to_be_served,Number_of_suppliers);
disp(' '); toc;  disp(' ')

% 7. Report

disp(' '); disp(' ')
disp(' Part 3. Report ')
disp(' ')

point_Pr_in_constraint = tbpsg_optimal_point_data(solution_Pr_in_constraint, output_Pr_in_constraint);
point_VaR_in_constraint = tbpsg_optimal_point_data(solution_VaR_in_constraint, output_VaR_in_constraint);
point_Pr_in_objective = tbpsg_optimal_point_data(solution_Pr_in_objective, output_Pr_in_objective);

point_vars_Pr_in_constraint = tbpsg_optimal_point_vars(solution_Pr_in_constraint, output_Pr_in_constraint);
point_vars_VaR_in_constraint = tbpsg_optimal_point_vars(solution_VaR_in_constraint, output_VaR_in_constraint);
point_vars_Pr_in_objective = tbpsg_optimal_point_vars(solution_Pr_in_objective, output_Pr_in_objective);


str=''; j=0;
for i=1:ks; if point_Pr_in_constraint(i)==1, str=[str '  ' point_vars_Pr_in_constraint{i}(2:end)]; j=j+1; if(j>=19),str=[str ' ...']; break; end; end; end
disp(['Solution_Pr_in_constraint.  Used suppliers: ' str]);

str=''; j=0;
for i=1:ks; if point_VaR_in_constraint(i)==1, str=[str '  ' point_vars_VaR_in_constraint{i}(2:end)]; j=j+1; if(j>=19),str=[str ' ...']; break; end; end; end
disp(['Solution_Var_in_constraint. Used suppliers: ' str]);

str=''; j=0;
for i=1:ks; if point_Pr_in_objective(i)==1, str=[str '  ' point_vars_Pr_in_objective{i}(2:end)]; j=j+1; if(j>=19),str=[str ' ...']; break; end; end; end
disp(['Solution_Pr_in_objective.   Used suppliers: ' str]);

clear str i j

disp(' ')
%===============================================================================================================================